C********************************************************************
C     asymrot.f
C
C     Dec. 5, 1993
C     J. Zmuidzinas
C
C     Program to calculate asymmetric rotor levels, transition
C     frequencies, degeneracies, and line strengths
C
C     Representation:
C        A = z-axis
C        B = x-axis
C        C = y-axis
C
C********************************************************************
C
      program asymrot
C
      call getconsts
      call hamdiag
      call writeoutput
      stop
      end
C
C********************************************************************
      subroutine getconsts
C********************************************************************
C
      character*80 infile
      include 'asymconst.inc'
C
      write(*,*) ' Molecular constants data file ? '
      read(*,1) infile
1     format(A)
C
      open(UNIT=1, FILE=infile, STATUS='OLD')
C
      read(1,*) jmax       ! maximum J to compute energies
      read(1,*) emax       ! maximum energy in K
      read(1,*) freqmax    ! maximum transition frequency in MHz
      read(1,*) strmin     ! minimum line strength
      read(1,*) ee_oo      ! flag to do k(-1) - k(+1) = ee or oo
      read(1,*) eo_oe      ! flag to do k(-1) - k(+1) = eo or oe
C
      read(1,*) adip       ! dipole moment along a axis in debye
      read(1,*) bdip       ! dipole moment along b axis in debye
      read(1,*) cdip       ! dipole moment along c axis in debye
C
      read(1,*) A          ! rotational constants
      read(1,*) B          ! see DeLucia, Helminger, Kirchoff 1974
      read(1,*) C          ! J. Phys. Chem. Ref. Data
      read(1,*) DJ         ! vol. 3, #1, 211-219
      read(1,*) DJK        ! gives definitions and also the
      read(1,*) DK         ! rotational constants for H2O
      read(1,*) DDJ
      read(1,*) DDK
      read(1,*) HJ
      read(1,*) HJK
      read(1,*) HKJ
      read(1,*) HK
      read(1,*) HHJ
      read(1,*) HHJK
      read(1,*) HHK
      read(1,*) LJ
      read(1,*) LJK
      read(1,*) LKKJ
      read(1,*) LK
      read(1,*) LLK
      read(1,*) PK
      read(1,*) PPK
C
      close(1)
      return
      end
C
C********************************************************************
      subroutine hamiltep(j, np, hep, dimhep)
C********************************************************************
C
C     ***calculate asymmetric rotor Hamiltonian for E+ block
C      **Inputs:
C        j = total angular momentum
C        np = physical dimension of hep matrix
C
C      **Outputs:
C        hep(np, np) = E+ hamiltonian
C        dimhep = actual dimension of hep for angular momentum j
C
C********************************************************************
C
      integer j, np, dimhep
      real*8 hep(np,np)
C
      include 'asymconst.inc'
      include 'asymham.inc'
C
      integer k1
      integer i1, i2
      real*8 xj, xk1, xk2
C
C     Calculate diagonal matrices first
C
      dimhep = j/2+1
      do 10 i1=1, dimhep
         xj = j*(j+1)
         p2(i1) = xj
         p4(i1) = p2(i1)*p2(i1)
         p6(i1) = p4(i1)*p2(i1)
         p8(i1) = p6(i1)*p2(i1)
C
         k1 = 2*(i1-1)
         pz2(i1) = k1*k1
         pz4(i1) = pz2(i1)*pz2(i1)
         pz6(i1) = pz4(i1)*pz2(i1)
         pz8(i1) = pz6(i1)*pz2(i1)
         pz10(i1) = pz8(i1)*pz2(i1)
C
C        now off-diagonal term
C
         if(i1.LT.dimhep) then      ! i1+1 still in range
            xk1 = k1*(k1+1)
            xk2 = (k1+1)*(k1+2)
            pm2(i1,i1+1) = 0.5*sqrt((xj-xk1)*(xj-xk2))
            if(i1.EQ.1) then      ! K=0 term 
               pm2(i1,i1+1) = pm2(i1,i1+1)*sqrt(2.0D0)
            endif
            pm2(i1+1,i1) = pm2(i1,i1+1)    ! matrix is symmetric
         endif
C
10    continue
C
C     Now calculate Hamiltonian
C
      do 30 i1=1, dimhep
         do 20 i2=1, dimhep
            hep(i1, i2) = 0.
20       continue
30    continue
C
      do 40 i1=1, dimhep
C
C        diagonal elements ...
C
         hep(i1,i1) = PK*pz10(i1) + LK*pz8(i1) + LKKJ*p2(i1)*pz6(i1)
     +              + LJK*p4(i1)*pz4(i1) + LJ*p8(i1)
     +              + HK*pz6(i1) + HKJ*p2(i1)*pz4(i1) 
     +              + HJK*p4(i1)*pz2(i1) + HJ*p6(i1)
     +              - DK*pz4(i1) - DJK*p2(i1)*pz2(i1) - DJ*p4(i1)
     +              + (A - 0.5*(B+C))*pz2(i1)
     +              + 0.5*(B+C)*p2(i1)
C
C        now the off-diagonals ...
C
         i2 = i1+1
         if(i2.LE.dimhep) then
            hep(i1,i2) = 
     +            PPK*(pz8(i1)*pm2(i1,i2) + pm2(i1,i2)*pz8(i2))
     +          + LLK*(pz6(i1)*pm2(i1,i2) + pm2(i1,i2)*pz6(i2))
     +          + HHK*(pz4(i1)*pm2(i1,i2) + pm2(i1,i2)*pz4(i2))
     +          + HHJK*p2(i1)*(pz2(i1)*pm2(i1,i2) + pm2(i1,i2)*pz2(i2))
     +          + 2.*HHJ*p4(i1)*pm2(i1,i2)
     +          - DDK*(pz2(i1)*pm2(i1,i2) + pm2(i1,i2)*pz2(i2))
     +          - 2.*DDJ*p2(i1)*pm2(i1,i2)
     +          + 0.5*(B-C)*pm2(i1,i2)
            hep(i2,i1) = hep(i1,i2)      ! matrix is symmetric
         endif
C
40    continue
      return
      end
C
C********************************************************************
      subroutine hamiltem(j, np, hem, dimhem)
C********************************************************************
C
C     ***calculate asymmetric rotor Hamiltonian for E- block
C      **Inputs:
C        j = total angular momentum
C        np = physical dimension of hem matrix
C
C      **Outputs:
C        hem(np, np) = E- hamiltonian
C        dimhem = actual dimension of hem for angular momentum j
C
C********************************************************************
C
      integer j, np, dimhem
      real*8 hem(np,np)
C
      include 'asymconst.inc'
      include 'asymham.inc'
C
      integer k1
      integer i1, i2
      real*8 xj, xk1, xk2
C
C     Calculate diagonal matrices first
C
      dimhem = j/2
      do 10 i1=1, dimhem
         xj = j*(j+1)
         p2(i1) = xj
         p4(i1) = p2(i1)*p2(i1)
         p6(i1) = p4(i1)*p2(i1)
         p8(i1) = p6(i1)*p2(i1)
C
         k1 = 2*i1
         pz2(i1) = k1*k1
         pz4(i1) = pz2(i1)*pz2(i1)
         pz6(i1) = pz4(i1)*pz2(i1)
         pz8(i1) = pz6(i1)*pz2(i1)
         pz10(i1) = pz8(i1)*pz2(i1)
C
C        now off-diagonal term
C
         if(i1.LT.dimhem) then      ! i1+1 still in range
            xk1 = k1*(k1+1)
            xk2 = (k1+1)*(k1+2)
            pm2(i1,i1+1) = 0.5*sqrt((xj-xk1)*(xj-xk2))
            pm2(i1+1,i1) = pm2(i1,i1+1)    ! matrix is symmetric
         endif
C
10    continue
C
C     Now calculate Hamiltonian
C
      do 30 i1=1, dimhem
         do 20 i2=1, dimhem
            hem(i1, i2) = 0.
20       continue
30    continue
C
      do 40 i1=1, dimhem
C
C
C        diagonal elements ...
C
         hem(i1,i1) = PK*pz10(i1) + LK*pz8(i1) + LKKJ*p2(i1)*pz6(i1)
     +              + LJK*p4(i1)*pz4(i1) + LJ*p8(i1)
     +              + HK*pz6(i1) + HKJ*p2(i1)*pz4(i1) 
     +              + HJK*p4(i1)*pz2(i1) + HJ*p6(i1)
     +              - DK*pz4(i1) - DJK*p2(i1)*pz2(i1) - DJ*p4(i1)
     +              + (A - 0.5*(B+C))*pz2(i1)
     +              + 0.5*(B+C)*p2(i1)
C
C        now the off-diagonals ...
C
         i2 = i1+1
         if(i2.LE.dimhem) then
            hem(i1,i2) = 
     +            PPK*(pz8(i1)*pm2(i1,i2) + pm2(i1,i2)*pz8(i2))
     +          + LLK*(pz6(i1)*pm2(i1,i2) + pm2(i1,i2)*pz6(i2))
     +          + HHK*(pz4(i1)*pm2(i1,i2) + pm2(i1,i2)*pz4(i2))
     +          + HHJK*p2(i1)*(pz2(i1)*pm2(i1,i2) + pm2(i1,i2)*pz2(i2))
     +          + 2.*HHJ*p4(i1)*pm2(i1,i2)
     +          - DDK*(pz2(i1)*pm2(i1,i2) + pm2(i1,i2)*pz2(i2))
     +          - 2.*DDJ*p2(i1)*pm2(i1,i2)
     +          + 0.5*(B-C)*pm2(i1,i2)
            hem(i2,i1) = hem(i1,i2)      ! matrix is symmetric
         endif
C
40    continue
      return
      end
C
C********************************************************************
      subroutine hamiltop(j, np, hop, dimhop)
C********************************************************************
C
C     ***calculate asymmetric rotor Hamiltonian for O+ block
C      **Inputs:
C        j = total angular momentum
C        np = physical dimension of hop matrix
C
C      **Outputs:
C        hop(np, np) = O+ hamiltonian
C        dimhop = actual dimension of hop for angular momentum j
C
C********************************************************************
C
      integer j, np, dimhop
      real*8 hop(np,np)
C
      include 'asymconst.inc'
      include 'asymham.inc'
C
      integer k1
      integer i1, i2
      real*8 xj, xk1, xk2
C
C     Calculate diagonal matrices first
C
      dimhop = (j+1)/2
      do 10 i1=1, dimhop
         xj = j*(j+1)
         p2(i1) = xj
         p4(i1) = p2(i1)*p2(i1)
         p6(i1) = p4(i1)*p2(i1)
         p8(i1) = p6(i1)*p2(i1)
C
         k1 = 2*(i1-1) +1
         pz2(i1) = k1*k1
         pz4(i1) = pz2(i1)*pz2(i1)
         pz6(i1) = pz4(i1)*pz2(i1)
         pz8(i1) = pz6(i1)*pz2(i1)
         pz10(i1) = pz8(i1)*pz2(i1)
C
C        now off-diagonal term
C
         if(i1.LT.dimhop) then      ! i1+1 still in range
            xk1 = k1*(k1+1)
            xk2 = (k1+1)*(k1+2)
            pm2(i1,i1+1) = 0.5*sqrt((xj-xk1)*(xj-xk2))
            pm2(i1+1,i1) = pm2(i1,i1+1)    ! matrix is symmetric
         endif
C
10    continue
C
C     pm2 matrix has diagonal term for K=1
C
      pm2(1,1) = 0.5*j*(j+1)   ! this is positive for O+ block
C
C
C     Now calculate Hamiltonian
C
      do 30 i1=1, dimhop
         do 20 i2=1, dimhop
            hop(i1, i2) = 0.
20       continue
30    continue
C
      do 40 i1=1, dimhop
C
C
C        diagonal elements ...
C
         hop(i1,i1) = PK*pz10(i1) + LK*pz8(i1) + LKKJ*p2(i1)*pz6(i1)
     +              + LJK*p4(i1)*pz4(i1) + LJ*p8(i1)
     +              + HK*pz6(i1) + HKJ*p2(i1)*pz4(i1) 
     +              + HJK*p4(i1)*pz2(i1) + HJ*p6(i1)
     +              - DK*pz4(i1) - DJK*p2(i1)*pz2(i1) - DJ*p4(i1)
     +              + (A - 0.5*(B+C))*pz2(i1)
     +              + 0.5*(B+C)*p2(i1)
C
C        now the off-diagonals ...
C
         i2 = i1+1
         if(i2.LE.dimhop) then
            hop(i1,i2) = 
     +            PPK*(pz8(i1)*pm2(i1,i2) + pm2(i1,i2)*pz8(i2))
     +          + LLK*(pz6(i1)*pm2(i1,i2) + pm2(i1,i2)*pz6(i2))
     +          + HHK*(pz4(i1)*pm2(i1,i2) + pm2(i1,i2)*pz4(i2))
     +          + HHJK*p2(i1)*(pz2(i1)*pm2(i1,i2) + pm2(i1,i2)*pz2(i2))
     +          + 2.*HHJ*p4(i1)*pm2(i1,i2)
     +          - DDK*(pz2(i1)*pm2(i1,i2) + pm2(i1,i2)*pz2(i2))
     +          - 2.*DDJ*p2(i1)*pm2(i1,i2)
     +          + 0.5*(B-C)*pm2(i1,i2)
            hop(i2,i1) = hop(i1,i2)      ! matrix is symmetric
         endif
C
40    continue
C
C     Add in contribution of special diagonal term of pm2
C
      hop(1,1) = hop(1,1)
     +          + PPK*(pz8(1)*pm2(1,1) + pm2(1,1)*pz8(1))
     +          + LLK*(pz6(1)*pm2(1,1) + pm2(1,1)*pz6(1))
     +          + HHK*(pz4(1)*pm2(1,1) + pm2(1,1)*pz4(1))
     +          + HHJK*p2(1)*(pz2(1)*pm2(1,1) + pm2(1,1)*pz2(1))
     +          + 2.*HHJ*p4(1)*pm2(1,1)
     +          - DDK*(pz2(1)*pm2(1,1) + pm2(1,1)*pz2(1))
     +          - 2.*DDJ*p2(1)*pm2(1,1)
     +          + 0.5*(B-C)*pm2(1,1)
C
      return
      end
C
C********************************************************************
      subroutine hamiltom(j, np, hom, dimhom)
C********************************************************************
C
C     ***calculate asymmetric rotor Hamiltonian for O- block
C      **Inputs:
C        j = total angular momentum
C        np = physical dimension of hom matrix
C
C      **Outputs:
C        hom(np, np) = O- hamiltonian
C        dimhom = actual dimension of hom for angular momentum j
C
C********************************************************************
C
      integer j, np, dimhom
      real*8 hom(np,np)
C
      include 'asymconst.inc'
      include 'asymham.inc'
C
      integer k1
      integer i1, i2
      real*8 xj, xk1, xk2
C
C     Calculate diagonal matrices first
C
      dimhom = (j+1)/2
      do 10 i1=1, dimhom
         xj = j*(j+1)
         p2(i1) = xj
         p4(i1) = p2(i1)*p2(i1)
         p6(i1) = p4(i1)*p2(i1)
         p8(i1) = p6(i1)*p2(i1)
C
         k1 = 2*(i1-1) +1
         pz2(i1) = k1*k1
         pz4(i1) = pz2(i1)*pz2(i1)
         pz6(i1) = pz4(i1)*pz2(i1)
         pz8(i1) = pz6(i1)*pz2(i1)
         pz10(i1) = pz8(i1)*pz2(i1)
C
C        now off-diagonal term
C
         if(i1.LT.dimhom) then      ! i1+1 still in range
            xk1 = k1*(k1+1)
            xk2 = (k1+1)*(k1+2)
            pm2(i1,i1+1) = 0.5*sqrt((xj-xk1)*(xj-xk2))
            pm2(i1+1,i1) = pm2(i1,i1+1)    ! matrix is symmetric
         endif
C
10    continue
C
C     pm2 matrix has diagonal term for K=1
C
      pm2(1,1) = -0.5*j*(j+1)   ! this is negative for O+ block
C
C
C     Now calculate Hamiltonian
C
      do 30 i1=1, dimhom
         do 20 i2=1, dimhom
            hom(i1, i2) = 0.
20       continue
30    continue
C
      do 40 i1=1, dimhom
C
C
C        diagonal elements ...
C
         hom(i1,i1) = PK*pz10(i1) + LK*pz8(i1) + LKKJ*p2(i1)*pz6(i1)
     +              + LJK*p4(i1)*pz4(i1) + LJ*p8(i1)
     +              + HK*pz6(i1) + HKJ*p2(i1)*pz4(i1) 
     +              + HJK*p4(i1)*pz2(i1) + HJ*p6(i1)
     +              - DK*pz4(i1) - DJK*p2(i1)*pz2(i1) - DJ*p4(i1)
     +              + (A - 0.5*(B+C))*pz2(i1)
     +              + 0.5*(B+C)*p2(i1)
C
C        now the off-diagonals ...
C
         i2 = i1+1
         if(i2.LE.dimhom) then
            hom(i1,i2) = 
     +            PPK*(pz8(i1)*pm2(i1,i2) + pm2(i1,i2)*pz8(i2))
     +          + LLK*(pz6(i1)*pm2(i1,i2) + pm2(i1,i2)*pz6(i2))
     +          + HHK*(pz4(i1)*pm2(i1,i2) + pm2(i1,i2)*pz4(i2))
     +          + HHJK*p2(i1)*(pz2(i1)*pm2(i1,i2) + pm2(i1,i2)*pz2(i2))
     +          + 2.*HHJ*p4(i1)*pm2(i1,i2)
     +          - DDK*(pz2(i1)*pm2(i1,i2) + pm2(i1,i2)*pz2(i2))
     +          - 2.*DDJ*p2(i1)*pm2(i1,i2)
     +          + 0.5*(B-C)*pm2(i1,i2)
            hom(i2,i1) = hom(i1,i2)      ! matrix is symmetric
         endif
C
40    continue
C
C     Add in contribution of special diagonal term of pm2
C
      hom(1,1) = hom(1,1)
     +          + PPK*(pz8(1)*pm2(1,1) + pm2(1,1)*pz8(1))
     +          + LLK*(pz6(1)*pm2(1,1) + pm2(1,1)*pz6(1))
     +          + HHK*(pz4(1)*pm2(1,1) + pm2(1,1)*pz4(1))
     +          + HHJK*p2(1)*(pz2(1)*pm2(1,1) + pm2(1,1)*pz2(1))
     +          + 2.*HHJ*p4(1)*pm2(1,1)
     +          - DDK*(pz2(1)*pm2(1,1) + pm2(1,1)*pz2(1))
     +          - 2.*DDJ*p2(1)*pm2(1,1)
     +          + 0.5*(B-C)*pm2(1,1)
C
      return
      end
C
C********************************************************************
      subroutine hamdiag
C********************************************************************
C
C     ***calculate and diagonalize asymmetric rotor Hamiltonian
C
      include 'asymconst.inc'
      include 'asymlev.inc'

      integer j, i1, i2, l1, k, k1, k2, jodd
      integer hdim, nrot
      real*8 sqr2    ! = sqrt(2.)
      real*8 x1
C
C
      sqr2 = sqrt(2.0D0)
C
C     Fill up energy array with -1's to indicate invalid
C     This is in case we only want ortho or para levels
C
      do 20 j=0, jmax
         do 10 l1=1, 2*jmax+1
            energy(j,l1) = -1.0
            do 5 k=-j,j
               evector(j,l1,k) = 0.    ! clear eigenvectors
5           continue
10       continue
20    continue
C
      if(ee_oo .NE. 0) then   ! do J=0
         energy(0,1) = 0.     ! which is trivial
         evector(0,1,0) = 1.    ! J=0 eigenvector is pure K=0 state
      endif
C
      do 110 j=1, jmax        ! calculate for all other J's
C
         jodd = j/2
         jodd = j - 2*jodd    ! =1 if j is odd; = 0 if j is even
C
C     ***Energy level ordering is E+, O-, O+, E-, E+, ...
C        For even J, ee=E+, oe=O-, oo=O+, eo=E-
C        For odd  J, eo=E+, oo=O-, oe=O+, ee=E-
C
         if( (jodd.EQ.0 .AND. ee_oo.NE.0) .OR.
     +       (jodd.NE.0 .AND. eo_oe.NE.0) )    then    ! go ahead with E+
C                                                        and O+ hams
C
            call hamiltep(j, MAXH, hmatrix, hdim)      ! calculate E+ ham.
C
C        ***now diagonalize with jacobi... (from numerical recipes)
C
            call DJACOBI(hmatrix, hdim, MAXH, henergy, hvect, nrot)
            call DEIGSRT(henergy, hvect, hdim, MAXH)    ! sort by energy
C
C        ***now stuff energies and eigenvectors away
C
            do 40 i1=1, hdim
               l1 = 4*(i1-1) + 1               ! level number
               energy(j, l1) = henergy(i1)
               x1 = hvect(1,i1)                ! handle K=0 separately
               evector(j,l1,0) = x1            ! because no root 2.
               do 30 i2=2, hdim
                  k = 2*(i2-1)                 ! E+ matrix has K=0,2,...
                  x1 = hvect(i2, i1)/sqr2
                  evector(j, l1, k)  = x1
                  evector(j, l1, -k) = x1
30             continue
40          continue
C
C        ***go on to O+ hamiltonian
C
            call hamiltop(j, MAXH, hmatrix, hdim)      ! calculate O+ ham.
C
C        ***now diagonalize with jacobi... (from numerical recipes)
C
            call DJACOBI(hmatrix, hdim, MAXH, henergy, hvect, nrot)
            call DEIGSRT(henergy, hvect, hdim, MAXH)    ! sort by energy
C
C        ***now stuff energies and eigenvectors away
C
            do 60 i1=1, hdim
               l1 = 4*(i1-1) + 3               ! level number
               energy(j, l1) = henergy(i1)
               do 50 i2=1, hdim
                  k = 2*(i2-1) +1              ! O+ matrix has K=1,3,...
                  x1 = hvect(i2, i1)/sqr2
                  evector(j, l1, k)  = x1
                  evector(j, l1, -k) = x1
50             continue
60          continue
         endif                                 ! done with E+, O+
C
C        See if we should do E- and O- blocks
C
         if( (jodd.EQ.0 .AND. eo_oe.NE.0) .OR.
     +       (jodd.NE.0 .AND. ee_oo.NE.0) )    then    ! go ahead with E-
C                                                        and O- hams
C
            if(j.GT.1) then                  ! no E- levels until J>=2
C
            call hamiltem(j, MAXH, hmatrix, hdim)      ! calculate E- ham.
C
C        ***now diagonalize with jacobi... (from numerical recipes)
C
            call DJACOBI(hmatrix, hdim, MAXH, henergy, hvect, nrot)
            call DEIGSRT(henergy, hvect, hdim, MAXH)    ! sort by energy
C
C        ***now stuff energies and eigenvectors away
C
            do 80 i1=1, hdim
               l1 = 4*i1                      ! level number
               energy(j, l1) = henergy(i1)
               do 70 i2=1, hdim
                  k = 2*i2                    ! E- matrix has K=2,4,...
                  x1 = hvect(i2, i1)/sqr2
                  evector(j, l1, k)  = x1
                  evector(j, l1, -k) = -x1    ! and opposite sign for -k
70             continue
80          continue
C
            endif      ! end if(j.GT.1)...
C
C        ***go on to O- hamiltonian
C
            call hamiltom(j, MAXH, hmatrix, hdim)      ! calculate O- ham.
C
C        ***now diagonalize with jacobi... (from numerical recipes)
C
            call DJACOBI(hmatrix, hdim, MAXH, henergy, hvect, nrot)
            call DEIGSRT(henergy, hvect, hdim, MAXH)    ! sort by energy
C
C        ***now stuff energies and eigenvectors away
C
            do 100 i1=1, hdim
               l1 = 4*(i1-1) + 2               ! level number
               energy(j, l1) = henergy(i1)
               do 90 i2=1, hdim
                  k = 2*(i2-1) +1              ! O- matrix has K=1,3,...
                  x1 = hvect(i2, i1)/sqr2
                  evector(j, l1, k)  = x1
                  evector(j, l1, -k) = -x1     ! and opposite sign for -k
90             continue
100         continue
         endif                                 ! done with E-, O-
110   continue
C
C     Generate standard labels for energy levels
C
      do 220 j=0, jmax
         k1 = 0
         k2 = j
         l1 = 1
         write(tmplabel, 200) j, k1, k2
200      format(I2, '(', I2, ',', I2, ')')
         elabel(j,l1) = tmplabel
C
         do 210 l1=2, 2*j+1, 2
            k1 = k1+1
            write(tmplabel, 200) j, k1, k2
            elabel(j,l1) = tmplabel
            k2 = k2-1
            write(tmplabel, 200) j, k1, k2
            elabel(j,l1+1) = tmplabel
210      continue
220   continue
C
C     Now need to sort energies of different J's
C
C    First, put valid energies into an array for sorting
C
      numlev = 0
      do 320 j=0, jmax
         do 310 l1=1, 2*j+1
            if(energy(j, l1) .GE. 0.) then    ! not = -1., so valid
               numlev = numlev+1
               esort(numlev) = energy(j,l1)
               jsort(numlev) = j             ! these indexes
               lsort(numlev) = l1            ! will be needed later
            endif
310      continue
320   continue
C
C     do the sort...
C
      call PIKSR3(numlev, esort, jsort, lsort)
C
C     We're done!
C
      return
      end
C
C********************************************************************
      subroutine writeoutput
C********************************************************************
C
      include 'asymconst.inc'
      include 'asymlev.inc'
      include 'cgs_constants.inc'
C
      integer lev, levi, levf, ji, li, ki, jf, lf, kf, iq
      integer nlevltmax
      real*8 totdip, strength, acoeff, wig3j, en, frq
      complex zmu(-1:1)     ! spherical components of dipole moment
      complex zstrength, zterm
C
      character*80 outfile
C
      real*4 vint3j         ! calculates Wigner 3-j coefficients
      external vint3j
C
      write(*,*) ' Output file name ? '
      read(*,1) outfile
1     format(A)
C
      open(UNIT=1, FILE=outfile, status='UNKNOWN')
C
C     Count how many levels below energy cutoff emax
C
      nlevltmax=0
5     continue
         nlevltmax = nlevltmax + 1
         en = esort(nlevltmax)*amegahz*hplanck/kboltz
      if(en .LT. emax .AND. nlevltmax .LT. numlev) goto 5
C
C     save this and level labels to a file
C
      write(1,*) ' Number of levels:'
      write(1,*) nlevltmax
      write(1,*) ' Level labels:'
      do 10 lev=1, nlevltmax
         ji = jsort(lev)
         li = lsort(lev)
         write(1,*) elabel(ji, li)
10    continue
C
      write(1,*) ' Level    Energy (K)   Degeneracy '
      do 20 lev=1, nlevltmax
         en = esort(lev)*amegahz*hplanck/kboltz
         write(1,*) lev, en, 2*jsort(lev)+1
20    continue
C
C     Calculate normalized spherical components of dipole moment
C
      totdip = sqrt(adip*adip + bdip*bdip + cdip*cdip)
      zmu(0) = adip/totdip
      zmu(1) = -cmplx(bdip, cdip)/(totdip*sqrt(2.0D0))
      zmu(-1) = cmplx(bdip, -cdip)/(totdip*sqrt(2.0D0))
C
      call setup3j                        ! calculate factorials for 3j's
C
C     Calculate transition line strengths
C
      write(1,*) ' I   F   A(I-F)   S(I-F)   Labels   freq(I-F)'
      do 60 levi=1, nlevltmax
         ji = jsort(levi)
         li = lsort(levi)
         do 50 levf = 1, levi-1
            jf = jsort(levf)
            lf = lsort(levf)
            frq = esort(levi) - esort(levf)
C
            if(iabs(jf-ji).LE.1 .AND.      ! allowed by dipole selection
     +         frq .LE. freqmax)   then    ! and frequency below cutoff
C
               zstrength = 0.              ! so calculate line strength
               do 40 iq=-1, 1              ! loop over zmu components
                  do 30 ki=-ji, ji
                     kf = ki - iq
                     if(iabs(kf).LE.jf) then   ! add it in
                        wig3j = vint3j(jf,1,ji,kf,iq,-ki)
                        zterm =
     +                    + zmu(iq)*evector(ji,li,ki)*evector(jf,lf,kf)
     +                     *((-1)**ki)*wig3j
                        zstrength = zstrength + zterm
                     endif
30                continue
40             continue
               strength = (2*ji+1)*(2*jf+1)*(cabs(zstrength)**2)
               if(strength.GT.strmin) then     ! calculate A-coeff
                  frq = frq*amegahz
                  acoeff = ( 32*PI*PI*PI/3 )
     +               *( (totdip*debye*frq/(echarge*clight))**2 )
     +               *( frq/alphafs )
     +               *( strength/(2*ji+1) )
                  write(1,45) levi, levf, acoeff, strength,
     +                    elabel(ji,li), ' - ', elabel(jf,lf),
     +                    frq/amegahz
45                format(1X, I3, 1X, I3, 1X, E11.4, 1X, F10.5,
     +                   1X, A10, A3, A10, 1X, F12.2)
               endif
            endif
50       continue
60    continue
C
      close(1)
      return
      end
C
C*************************************************************************
C      Subroutines to calculate three-j symbols
C
C      J. Zmuidzinas, Dec. 5 1993
C
C      Copied from 'Angular Momentum' by R. Zare, Appendix A
C      with minor mods
C
C*************************************************************************
      subroutine setup3j
C*************************************************************************
C
C     ***calculates and stores factorials
C
      implicit real*8 (A-H, O-Z)
      parameter (Nmax=400)
      dimension facl(Nmax)
      common/factor/facl
C
      facl(1)=0.D0
      do 100 i=2, Nmax
         i1 = i-1
         facl(i) = facl(i1) + dlog(dfloat(i1))
100   continue
      return
      end
C
C*************************************************************************
      real*8 function frontl(x1, x2, x3, y1, y2, y3)
C*************************************************************************
C
C     ***used in three-j calculation
C
      implicit real*8 (A-H, O-Z)
      parameter (Nmax=400)
      dimension fl(Nmax)
      common/factor/fl
C
      l1 = ifix(sngl(x1+x2-x3+1.1))
      l2 = ifix(sngl(x2+x3-x1+1.1))
      l3 = ifix(sngl(x3+x1-x2+1.1))
      l4 = ifix(sngl(x1+x2+x3+2.1))
      l5 = ifix(sngl(x1+y1+1.1))
      l6 = ifix(sngl(x1-y1+1.1))
      l7 = ifix(sngl(x2+y2+1.1))
      l8 = ifix(sngl(x2-y2+1.1))
      l9 = ifix(sngl(x3+y3+1.1))
      l10 = ifix(sngl(x3-y3+1.1))
C
      frontl = fl(l1) + fl(l2) + fl(l3) - fl(l4) + fl(l5) +fl(l6)
      frontl = frontl + fl(l7) + fl(l8) + fl(l9) + fl(l10)
      frontl = frontl/2.D0
C
      return
      end
C
C*************************************************************************
      real*4 function real3j(rj1, rj2, rj3, rm1, rm2, rm3)
C*************************************************************************
C
C     ***calculates 3-j coefficient
C        real3j - each real argument is actual value
C
C*************************************************************************
C
      implicit real*8 (A-H, O-Z)
      parameter (Nmax=400)
      dimension fl(Nmax)
      common/factor/fl
      real*8 frontl
      external frontl
C
      real*4 rj1, rj2, rj3, rm1, rm2, rm3
C
      j1d = int(2*rj1)
      j2d = int(2*rj2)
      j3d = int(2*rj3)
      m1d = int(2*rm1)
      m2d = int(2*rm2)
      m3d = int(2*rm3)
C
      x1 = dfloat(j1d)/2.D0
      x2 = dfloat(j2d)/2.D0
      x3 = dfloat(j3d)/2.D0
      y1 = dfloat(m1d)/2.D0
      y2 = dfloat(m2d)/2.D0
      y3 = dfloat(m3d)/2.D0
C
      if(iabs(m1d).GT.j1d) goto 9998
      if(iabs(m2d).GT.j2d) goto 9998
      if(iabs(m3d).GT.j3d) goto 9998
C
      if(j1d+j2d-j3d.LT.0) goto 9998
      if(j2d+j3d-j1d.LT.0) goto 9998
      if(j3d+j1d-j2d.LT.0) goto 9998
C
      if(m1d+m2d+m3d.NE.0) goto 9998
C
      kmin = (j3d-j1d-m2d)/2
      kmin1 = kmin
      kmin2 = (j3d-j2d+m1d)/2
      if(kmin2.LT.kmin) kmin = kmin2
      kmin = (-1)*kmin
C
      kmax = ifix(sngl(x1 + x2 - x3 + 0.1))
      kmax1 = kmax
      kmax2 = ifix(sngl(x1 - y1 + 0.1))
      kmax3 = ifix(sngl(x2 + y2 + 0.1))
      if(kmax2.LT.kmax) kmax = kmax2
      if(kmax3.LT.kmax) kmax = kmax3
C
      if(kmin.LT.0) kmin=0
      if(kmin.GT.kmax) goto 9998
C
      jmin = kmin+1
      jmax = kmax+1
      term1 = frontl(x1, x2, x3, y1, y2, y3)
      msign = (-1)**((j1d-j2d-m3d)/2)
      sum = 0.0D0
C
      do 10 i1=jmin, jmax
         i = i1-1
         term2 = fl(i1) + fl(kmin1+i1) + fl(kmin2+i1)
         term2 = term2 + fl(kmax1-i+1) + fl(kmax2-i+1) + fl(kmax3-i+1)
         term = dexp(term1-term2)
         term = term*msign*((-1)**i)
         sum = sum + term
10    continue
C
      goto 9999
C
9998  continue
      sum = 0.0
9999  continue
      real3j = sum
C
      return
      end
C
C*************************************************************************
      real*4 function thrj(jj1, jj2, jj3, mm1, mm2, mm3)
C*************************************************************************
C
C     ***calculates 3-j coefficient
C        thrj - each integer argument is 2x actual value
C
C*************************************************************************
C
      real*4 real3j
      external real3j
      real*4 result, xj1, xj2, xj3, xm1, xm2, xm3
C
      xj1 = jj1/2.0D0
      xj2 = jj2/2.0D0
      xj3 = jj3/2.0D0
      xm1 = mm1/2.0D0
      xm2 = mm2/2.0D0
      xm3 = mm3/2.0D0
      result = real3j(xj1, xj2, xj3, xm1, xm2, xm3)
      thrj = result
      return
      end
C
C*************************************************************************
      real*4 function vint3j(ij1, ij2, ij3, im1, im2, im3)
C*************************************************************************
C
C     ***calculates 3-j coefficient
C        vint3j - each integer argument is actual value (no 1/2 ints)
C
C*************************************************************************
C
      real*4 real3j
      external real3j
      real*4 result, xj1, xj2, xj3, xm1, xm2, xm3
C
      xj1 = ij1
      xj2 = ij2
      xj3 = ij3
      xm1 = im1
      xm2 = im2
      xm3 = im3
      result = real3j(xj1, xj2, xj3, xm1, xm2, xm3)
      vint3j = result
      return
      end
C
C********************************************************************
      SUBROUTINE PIKSR3(N,ARR,BRR,CRR)
C********************************************************************
      REAL*8 ARR(N), A
      INTEGER BRR(N), CRR(N), B, C
      DO 12 J=2,N
        A=ARR(J)
        B=BRR(J)
        C=CRR(J)
        DO 11 I=J-1,1,-1
          IF(ARR(I).LE.A)GO TO 10
          ARR(I+1)=ARR(I)
          BRR(I+1)=BRR(I)
          CRR(I+1)=CRR(I)
11      CONTINUE
        I=0
10      ARR(I+1)=A
        BRR(I+1)=B
        CRR(I+1)=C
12    CONTINUE
      RETURN
      END
C
C********************************************************************
      SUBROUTINE DJACOBI(A,N,NP,D,V,NROT)
C********************************************************************
C
C     ***double precision version of this routine***
C
      IMPLICIT REAL*8 (A-H, O-Z)
C
      PARAMETER (NMAX=100)
      DIMENSION A(NP,NP),D(NP),V(NP,NP),B(NMAX),Z(NMAX)
      DO 12 IP=1,N
        DO 11 IQ=1,N
          V(IP,IQ)=0.
11      CONTINUE
        V(IP,IP)=1.
12    CONTINUE
      DO 13 IP=1,N
        B(IP)=A(IP,IP)
        D(IP)=B(IP)
        Z(IP)=0.
13    CONTINUE
      NROT=0
      DO 24 I=1,50
        SM=0.
        DO 15 IP=1,N-1
          DO 14 IQ=IP+1,N
            SM=SM+ABS(A(IP,IQ))
14        CONTINUE
15      CONTINUE
        IF(SM.EQ.0.)RETURN
        IF(I.LT.4)THEN
          TRESH=0.2*SM/N**2
        ELSE
          TRESH=0.
        ENDIF
        DO 22 IP=1,N-1
          DO 21 IQ=IP+1,N
            G=100.*ABS(A(IP,IQ))
            IF((I.GT.4).AND.(ABS(D(IP))+G.EQ.ABS(D(IP)))
     *         .AND.(ABS(D(IQ))+G.EQ.ABS(D(IQ))))THEN
              A(IP,IQ)=0.
            ELSE IF(ABS(A(IP,IQ)).GT.TRESH)THEN
              H=D(IQ)-D(IP)
              IF(ABS(H)+G.EQ.ABS(H))THEN
                T=A(IP,IQ)/H
              ELSE
                THETA=0.5*H/A(IP,IQ)
                T=1./(ABS(THETA)+SQRT(1.+THETA**2))
                IF(THETA.LT.0.)T=-T
              ENDIF
              C=1./SQRT(1+T**2)
              S=T*C
              TAU=S/(1.+C)
              H=T*A(IP,IQ)
              Z(IP)=Z(IP)-H
              Z(IQ)=Z(IQ)+H
              D(IP)=D(IP)-H
              D(IQ)=D(IQ)+H
              A(IP,IQ)=0.
              DO 16 J=1,IP-1
                G=A(J,IP)
                H=A(J,IQ)
                A(J,IP)=G-S*(H+G*TAU)
                A(J,IQ)=H+S*(G-H*TAU)
16            CONTINUE
              DO 17 J=IP+1,IQ-1
                G=A(IP,J)
                H=A(J,IQ)
                A(IP,J)=G-S*(H+G*TAU)
                A(J,IQ)=H+S*(G-H*TAU)
17            CONTINUE
              DO 18 J=IQ+1,N
                G=A(IP,J)
                H=A(IQ,J)
                A(IP,J)=G-S*(H+G*TAU)
                A(IQ,J)=H+S*(G-H*TAU)
18            CONTINUE
              DO 19 J=1,N
                G=V(J,IP)
                H=V(J,IQ)
                V(J,IP)=G-S*(H+G*TAU)
                V(J,IQ)=H+S*(G-H*TAU)
19            CONTINUE
              NROT=NROT+1
            ENDIF
21        CONTINUE
22      CONTINUE
        DO 23 IP=1,N
          B(IP)=B(IP)+Z(IP)
          D(IP)=B(IP)
          Z(IP)=0.
23      CONTINUE
24    CONTINUE
      PAUSE '50 iterations should never happen'
      RETURN
      END
C
C********************************************************************
      SUBROUTINE DEIGSRT(D,V,N,NP)
C********************************************************************
C
C     ***double precision version of this routine***
C     ***also changed to sort in ASCENDING order ***
C
      IMPLICIT REAL*8 (A-H, O-Z)
C
      DIMENSION D(NP),V(NP,NP)
      DO 13 I=1,N-1
        K=I
        P=D(I)
        DO 11 J=I+1,N
          IF(D(J).LE.P)THEN
            K=J
            P=D(J)
          ENDIF
11      CONTINUE
        IF(K.NE.I)THEN
          D(K)=D(I)
          D(I)=P
          DO 12 J=1,N
            P=V(J,I)
            V(J,I)=V(J,K)
            V(J,K)=P
12        CONTINUE
        ENDIF
13    CONTINUE
      RETURN
      END
